#!/usr/bin/perl
#
# ARCHMBOX: archives old messages from a mailbox.
# Copyright (C) 2001 Alessandro Dotti
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# For any questions related to this software, please write me at:
#
# Alessandro Dotti
# v. Verne, 6
# 40128 Bologna ITALY
#
# or email me at: alessandro.dotti@libero.it
#==============================================================================
# Archive all messages older than the specified date from
# the specified mailbox.
# Archived messages are appended to the file <mailbox>.archived
#
# usage: archmbox [-h|-v]
#        archmbox [OPTIONS] -d <date> <mailbox>
#	 archmbox [OPTIONS] -o <days> <mailbox>
#
# OPTIONS
#	-d, --date <date>:		treshold date for messages
#	-e, --extension <extension>:	suffix for the archive mailbox (default: "archived")
#	-h, --help:			prints help
#	-o, --offset <days>:		offset (in days) from today for treshold date
#	-p, --path <directory>:		where to store archive mailbox (default: ".")
#	-v, --version:			prints version number
#
# <date> must be supplied using the following format: yyyy-mm-dd
# <mailbox> must be a regular mbox file.
#==============================================================================

# Shell helpers
#==============================================================================

# Where are the helpers I need located?

$FUSER = "/bin/fuser";
$RM = "/bin/rm";
$MV = "/bin/mv";

# Modules
#==============================================================================

use File::Basename;
use Getopt::Long;
use Time::Local;

# Functions
#==============================================================================

sub print_help();		# Print help
sub print_version();		# Print version
sub date_from_offset($);	# Get a date from an offset

# Constants
#==============================================================================

# Last day of month

my %LAST_DAY = (	"01" => 31, "02" => 29, "03" => 31, "04" => 30,
			"05" => 31, "06" => 30,	"07" => 31, "08" => 31,
			"09" => 30, "10" => 31, "11" => 30, "12" => 31
);

# Month convertion from char to digit
# Since the return value must be use with timelocal, the count starts
# from zero

my %MONTH_TO_DIGIT = (	Jan =>  0, Feb =>  1, Mar =>  2, Apr =>  3,
			May =>  4, Jun =>  5, Jul =>  6, Aug =>  7,
			Sep =>  8, Oct =>  9, Nov => 10, Dec => 11
);

my $VERSION = "0.3.2";	# Script version

# Parameters checking
#==============================================================================

# Options parsing

my $DATE;		# Threshold date for messages
my $EXTENSION;		# Suffix for the archive mailbox
my $PRINT_HELP;		# Print help
my $OFFSET;		# Offset (in days) from today for treshold date
my $PATH;		# where to store archive mailbox
my $PRINT_VERSION;	# Print version number

unless ( GetOptions (
		"d=s"		=> \$DATE,
		"date=s"	=> \$DATE,
		"e=s"		=> \$EXTENSION,
		"extension=s"	=> \$EXTENSION,
		"h"		=> \$PRINT_HELP,
		"help"		=> \$PRINT_HELP,
		"o=i"		=> \$OFFSET,
		"offset=i"	=> \$OFFSET,
		"p=s"		=> \$PATH,
		"path=s"	=> \$PATH,
		"v"		=> \$PRINT_VERSION,
		"version"	=> \$PRINT_VERSION
	))
	{ print_help(); exit 1; }

print_help() if $PRINT_HELP;		# Print help
print_version() if $PRINT_VERSION;	# Print version

# Check for mandatory options

unless ($DATE xor $OFFSET) { die "You must specify a date (-d) OR an offset (-o)\n"; }

# Check for parameters

if ((scalar @ARGV) == 0) { die "Missing mailbox.\n"; }

if ((scalar @ARGV) > 1) { die "Wrong number of parameters.\n";}

# Get mailbox

my $MBOX = $ARGV[0];

unless (-f $MBOX) { die "No valid mailbox supplied.\n"; }

# Get date

$DATE = date_from_offset($OFFSET) if $OFFSET;	# Treshold date specified as an offset

my $YEAR; my $MONTH; my $DAY;

if ($DATE =~ /(\d{4})-(\d{2})-(\d{2})/)
{
	if (($2 < 01) || ($2 > 12)) { die "Month is not valid.\n"; }
	if (($3 < 01) || ($3 > $LAST_DAY{$2})) { die "Day is not valid.\n"; }

	$YEAR = $1;
	$MONTH = $2;
	$DAY = $3;
}
else { die "Date was not specified in the right format!\n"; }	# Incorrect date format

# Check other options

if ($PATH)
{
	unless ($PATH) { die "The specified path is not valid... $PATH: $!\n"; }
}

# Suffix for the archive mailbox

$EXTENSION = "archived" unless $EXTENSION;

# Remove and archive messages
#==============================================================================

# Treshold date for messages age

my $TRESHOLD_DATE = timelocal(0, 0, 0, $DAY, ($MONTH - 1), ($YEAR - 1900));

my $messages = 0;	# Messages counter
my $removed = 0;	# Removed messages counter

# Check if the mailbox is in use

system("$FUSER -s $MBOX") or die "Mailbox $MBOX is in use. Better quit!\n";

# Open files

my $MBOX_NAME = basename($MBOX);
my $NEW_MBOX = "$MBOX.kept";
my $OLD_MBOX = ($PATH ? "$PATH/$MBOX_NAME.$EXTENSION" : "$MBOX_NAME.$EXTENSION");

# Get some info about the mailbox

my ($mode, $uid, $gid, $mtime) = (stat $MBOX)[2, 4, 5, 9];

open (MBOX, $MBOX) or die "Unable to open MBOX: $!\n";
open (NEWMBOX, ">$NEW_MBOX") or die "Unable to create $NEW_MBOX: $!\n";
open (OLDMBOX, ">>$OLD_MBOX") or die "Unable to create $OLD_MBOX: $!\n";

# Parse mailbox

my $ARCHIVE = 0;	# Archive mode for messages

while (<MBOX>)
{
        chomp (my $line = $_);

	if ($line =~ /^From\s.*\@.*\s/)
	{
		# This is a new message
		$messages++;
	  	my ($month, $day, $year) = (split (" ", $line))[3,4,6];

		my $message_date = timelocal(0, 0, 0, $day, $MONTH_TO_DIGIT{$month}, ($year - 1900));

		if ($message_date < $TRESHOLD_DATE)
		{
			# The message is old
			$removed++;
			$ARCHIVE = 1;
		}
		else { $ARCHIVE = 0; }
	}
	print { $ARCHIVE ? OLDMBOX : NEWMBOX } "$line\n";
}

# Print results

print "Mailbox $MBOX: archived $removed messages out of $messages\n";

# Close files

close (MBOX);
close (NEWMBOX);
close (OLDMBOX);

# Update mailbox and exit

# Is the mailbox in use now or was it modified during operations?

my $not_used = system("$FUSER -s $MBOX");
my $not_modified = ($mtime == (stat $MBOX)[9]) ? 1: 0;

unless ($not_used and $not_modified)
{
	print "Mailbox $MBOX was modified or is in use now.\n";
	print "It's not safe to complete the operation... I quit.\n";

	# Remove mailboxes

	qx{$RM $NEW_MBOX $OLD_MBOX};

	exit 1;
}

qx{$MV $NEW_MBOX $MBOX};

# Restore mailbox ownership and permissions

chown $uid, $gid, $MBOX, $OLD_MBOX;
chmod $mode, $MBOX, $OLD_MBOX;

exit 0;

# Functions
#==============================================================================

sub print_help()
{
# Print help

print <<__HELP__;
Archmbox:
archive all messages older than the specified date from
the specified mailbox.
Archived messages are appended to the file <mailbox>.archived.

usage: archmbox [-h|-v]
       archmbox [OPTIONS] -d <date> <mailbox>
       archmbox [OPTIONS] -o <days> <mailbox>

OPTIONS
  -d, --date <date>:		treshold date for messages
  -e, --extension <extension>:	suffix for the archive mailbox
  				(default: "archived")
  -h, --help:			prints help
  -o, --offset <days>:		offset (in days) from today for treshold date
  -p, --path <directory>:	where to store archive mailbox (default: ".")
  -v, --version:		prints version number

<date> must be supplied using the following format: yyyy-mm-dd
<mailbox> must be a regular mbox file.
__HELP__

exit 0;
}

sub print_version()
{
# Print script version

print "$VERSION\n";
exit 0;
}

sub date_from_offset($)
{
# Return the date as today - offset

my $SECS_X_DAY = 86400;	# Seconds in a day

my $offset = abs(shift @_);
my $offset_secs = $offset * $SECS_X_DAY;
my ($mday, $mon, $year, $yday) = (localtime(time - $offset_secs))[3,4,5,7];
	
$year += 1900;
$mon += 1;
$mon = "0" . $mon if ($mon =~ /^\d$/);
$mday = "0" . $mday if ($mday =~ /^\d$/);
return "$year-$mon-$mday";
}
